<?php

/**
 * @file
 * Contains \Drupal\leaflet_countries\Countries.
 */

namespace Drupal\leaflet_countries;

/**
 * Class Countries
 * @package Drupal\leaflet_countries
 *
 * Contains helper methods for accessing GeoJSON library data
 * and processing the data with GeoPHP.
 */
class Countries {

  /**
   * Loads the countries JSON file from the filesystem.
   *
   * @return string
   *   The contents of the JSON file.
   */
  protected static function getCountriesJSON() {
    $library = \Drupal::service('library.discovery')->getLibraryByName('leaflet_countries', 'leaflet_countries.countries_list');
    $path = \Drupal::service('file_system')->realpath($library['js'][0]['data']);
    $file = file_get_contents($path);
    return $file;
  }

  /**
   * Decodes the countries JSON data and returns it.
   */
  protected static function getCountries() {
    $countries = json_decode(self::getCountriesJSON());
    return $countries;
  }

  /**
   * Gets the country codes and returns an array of them.
   *
   * @return array
   *   A simple array of country codes.
   */
  public static function getCodes() {
    $list = self::getCodesAndLabels();
    $codes = array();
    foreach($list as $code => $name) {
      $codes[] = $code;
    }
    return $codes;
  }

  /**
   * Gets the country codes and labels and returns a keyed value pair.
   *
   * @return array
   *   An array with the key being the country code and the value being
   *   the country's common name.
   */
  public static function getCodesAndLabels() {
    $list = self::getCountries();
    $labels = array();
    foreach($list as $item) {
      $labels[$item->cca3] = $item->name->common;
    }
    asort($labels);
    return $labels;
  }

  /**
   * Gets an individual country's JSON data.
   *
   * @param $code
   *   The country code of the country whose data should be returned.
   */
  public static function getIndividualCountry($code) {
    $filepath = 'libraries/countries/data/' . strtolower($code) . '.geo.json';
    $path = \Drupal::service('file_system')->realpath($filepath);
    $content = file_get_contents($path);
    return $content;
  }

  /**
   * Processes the GeoJSON data using the GeoPHP library.
   *
   * @param $items
   *   The GeoJSON data.
   *
   * @return array
   *   A set of data that can be passed to leaflet as a layer.
   */
  public static function processMultiPolygon($items) {
    if (!is_array($items)) {
      $items = array($items);
    }
    $data = array();
    foreach ($items as $item) {
      if (!($geom = \Drupal::service('leaflet_countries.geophp')->load($item))) {
        continue;
      }
      $datum = array('type' => strtolower($geom->geometryType()));

      switch ($datum['type']) {
        case 'polygon':
          $tmp = $geom->getComponents();
          $components = $tmp[0]->getComponents();
          foreach ($components as $component) {
            $datum['points'][] = array(
              'lat' => $component->getY(),
              'lon' => $component->getX(),
            );
          }
          break;

        case 'multipolygon':
          $tmp = $geom->getComponents();
          foreach ($tmp as $delta => $polygon) {
            $polygon_component = $polygon->getComponents();
            foreach ($polygon_component as $delta => $linestring) {
              $components[] = $linestring;
            }
          }
          foreach ($components as $key => $component) {
            $subcomponents = $component->getComponents();
            foreach ($subcomponents as $subcomponent) {
              $datum['component'][$key]['points'][] = array(
                'lat' => $subcomponent->getY(),
                'lon' => $subcomponent->getX(),
              );
            }
            unset($subcomponent);
          }
          break;

      }

      $data[] = $datum;
    }

    return $data;
  }

}
